// file: /u1/picone/jeida/excise_signal/excise_signal.cc
//
// this is a simple program that excises a signal.
//

// system include files
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// isip include files
//
#include <math/integral/integral.h>
#include <math/integral/integral_constants.h>

// local include files
//
#include "excise_signal.h"

// main program
//
main (int argc, char** argv) {

  // variables for command line arguments
  //
  float_4 sample_freq = DEFAULT_SAMPLE_FREQ;
  float_4 start_time = DEFAULT_START_TIME;
  float_4 end_time = DEFAULT_END_TIME;
  int_4 num_bytes = DEFAULT_NUMBER_BYTES_PER_SAMPLE;
  int_4 num_chan = DEFAULT_NUMBER_OF_CHANNELS;
  char_1 file_in[999];
  char_1 file_out[999];

  // parse the command line
  //
  if (get_parameters_C(argc, argv, sample_freq,
		       start_time, end_time,
		       num_bytes, num_chan,
		       file_in, file_out) == ISIP_FALSE) {
    fprintf(stdout,
	    "%s: error in command line arguments\n",
	    ISIP_PROGRAM_NAME);

    return(ISIP_FATAL_ERROR);
  }

  // open the file to be read
  //
  FILE *fp_in = fopen((char*)file_in, "r");

  if (fp_in == NULL) {
    fprintf(stdout,
	    "%s: error opening file [%s]\n",
	    ISIP_PROGRAM_NAME, file_in);

    return(ISIP_FATAL_ERROR);
  }

  // open the file to be written
  //
  FILE *fp_out = fopen((char*)file_out, "w");

  if (fp_out == NULL) {
    fprintf(stdout,
	    "%s: error opening file [%s]\n",
	    ISIP_PROGRAM_NAME, file_out);

    return(ISIP_FATAL_ERROR);
  }

  // compute the end of file if necessary
  //
  fseek(fp_in, 0, SEEK_END);
  int_4 file_size = ftell(fp_in);
  float_4 end_file_time = (float_4)file_size / 
    (sample_freq * (float_4)num_bytes * (float_4)num_chan);
  rewind(fp_in);

  if ((end_time < start_time) || (end_time > end_file_time)) {
    fprintf(stdout, "%s: end_time (%f) invalid (%f %f)\n",
	    ISIP_PROGRAM_NAME, end_time, start_time, end_file_time);
    fclose(fp_in);
    fclose(fp_out);
    return(ISIP_FATAL_ERROR);
  }

  // compute an i/o buffer size
  //
  int_4 num_samples = (int_4)((end_time - start_time) * sample_freq);

  int_4 buf_size = DEFAULT_BUFFER_SIZE;
  if (num_samples < buf_size)
    {buf_size = num_samples;}

  // allocate an i/o buffer
  //
  char_1* buffer = (char_1*)malloc(buf_size*num_chan*num_bytes);

  // initialize counters
  //
  int_4 i_cur_time = (int_4)(start_time * sample_freq);
  int_4 i_end_time = (int_4)(end_time * sample_freq);

  // seek to the starting point in the file
  //
  fseek(fp_in, i_cur_time*num_bytes*num_chan, SEEK_SET);

  // main loop
  //
  while (i_cur_time < i_end_time) {

    // read data
    //
    int_4 samples_read = fread(buffer, num_chan*num_bytes,
			       buf_size, fp_in);

    if((i_cur_time + samples_read) > i_end_time)
      {samples_read = i_end_time - i_cur_time;}

    // output the data
    //
    int_4 samples_writ = fwrite(buffer, num_chan*num_bytes,
				samples_read, fp_out);
    if (samples_writ != samples_read) {
      fprintf(stdout, "%s: samples_writ (%ld) != samples_read (%ld)\n",
	      ISIP_PROGRAM_NAME, samples_writ, samples_read);
      fclose(fp_in);
      fclose(fp_out);
      return(ISIP_FATAL_ERROR);
    }

    // update counters
    //
    i_cur_time += samples_read;
  }

  // close the files
  //
  fclose(fp_in);
  fclose(fp_out);

  // exit gracefully
  //
  return(ISIP_NO_ERROR);
}
