// file: /u1/picone/jeida/plot_signal/plot_signal.cc
//
// this is a simple program that prints signal values to stdout
// using a standard interpolation algorithm
//

// system include files
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// isip include files
//
#include <math/integral/integral.h>
#include <math/integral/integral_constants.h>

// local include files
//
#include "plot_signal.h"

// main program
//
main (int argc, char** argv) {

  // variables for command line arguments
  //
  float_4 sample_freq = DEFAULT_SAMPLE_FREQ;
  float_4 start_time = DEFAULT_START_TIME;
  float_4 end_time = DEFAULT_END_TIME;
  int_4 resolution = DEFAULT_RESOLUTION;
  int_4 num_bytes = DEFAULT_NUMBER_BYTES_PER_SAMPLE;
  int_4 num_chan = DEFAULT_NUMBER_OF_CHANNELS;
  int_4 chan = DEFAULT_CHANNEL_NUMBER;
  char_1 file[999];

  // parse the command line
  //
  if (get_parameters_C(argc, argv, 
		       sample_freq, start_time, end_time, resolution,
		       num_bytes, num_chan, chan, file) == ISIP_FALSE) {
    fprintf(stdout,
	    "%s: error in command line arguments\n",
	    ISIP_PROGRAM_NAME);

    return(ISIP_FATAL_ERROR);
  }

  // open the file to be plotted
  //
  FILE *fp = fopen((char*)file, "r");

  if (fp == NULL) {
    fprintf(stdout,
	    "%s: error opening file [%s]\n",
	    ISIP_PROGRAM_NAME, file);

    return(ISIP_FATAL_ERROR);
  }

  // compute the end of file if necessary
  //
  if (end_time < start_time) {
    fseek(fp, 0, SEEK_END);
    int_4 file_size = ftell(fp);
    end_time = (float_4)file_size / (sample_freq *
				     (float_4)num_bytes * (float_4)num_chan);
    rewind(fp);
  }

  // compute an i/o buffer size
  //
  int_4 num_samples = (int_4)((end_time - start_time) * sample_freq);
  int_4 buf_size = DEFAULT_BUFFER_SIZE;

  if (num_samples*num_chan < buf_size)
    {buf_size = num_samples;}

  // compute the decimation factor and adjust the buf_size
  //
  int_4 decimation_factor = num_samples / resolution;
  if (decimation_factor < (int_4)1)
    {decimation_factor = (int_4)1;}
  if (buf_size % decimation_factor != 0) {
    buf_size += decimation_factor;
    buf_size /= decimation_factor;
    buf_size *= decimation_factor;
  }

  // allocate an i/o buffer
  //
  char_1* buffer = (char_1*)malloc(buf_size*num_chan*num_bytes);

  // initialize counters
  //
  int_4 i_cur_time = (int_4)(start_time * sample_freq);
  int_4 i_end_time = (int_4)(end_time * sample_freq);
  int_4 i_output_time = i_cur_time + decimation_factor - (int_4)1;

  // seek to the starting point in the file
  //
  fseek(fp, i_cur_time*num_bytes*num_chan, SEEK_SET);

  // initialize parameters to find min/max
  //
  int_4 sample_val;
  int_4 min_val = DEFAULT_LARGE_NUMBER;
  int_4 min_loc = (int_4)0;
  int_4 max_val = -min_val;
  int_4 max_loc = (int_4)0;

  // main loop
  //
  while (i_cur_time < i_end_time) {

    // read data
    //
    fread(buffer, num_chan*num_bytes, buf_size, fp);

    // decimate the signal
    //
    for (int_4 i=0; i<buf_size; i++) {

      // compute the sample value
      //
      if (num_bytes == (int_4)2) {
	sample_val = *((int_2*)(buffer + i*(num_chan*num_bytes) +
				chan*num_bytes));
      }
      else if (num_bytes == (int_4)4) {
	sample_val = *((int_4*)(buffer + i*(num_chan*num_bytes) +
				chan*num_bytes));
      }
      else {
	fprintf(stdout,
		"%s: error in sample format [%ld]\n",
		ISIP_PROGRAM_NAME, num_bytes);
	return(ISIP_FATAL_ERROR);
      }

      // check for min/max
      //
      if (sample_val < min_val) {
	min_val = sample_val;
	min_loc = i_cur_time;
      }

      if (sample_val > max_val) {
	max_val = sample_val;
	max_loc = i_cur_time;
      }
//      printf("%d %d (%d): %d %d %d %d\n",i_cur_time, i_output_time,
//	     sample_val, min_val, min_loc, max_val, max_loc);

      // output the critical points if necessary
      //
      if ((i_cur_time == i_output_time) && (i_cur_time < i_end_time)) {

	if (min_loc < i_end_time) {
	  printf("%f %f\n",
		 (float_4)(min_loc) / sample_freq, (float_4)min_val);
	}

	if (max_loc < i_end_time) {
	  printf("%f %f\n",
		 (float_4)(max_loc) / sample_freq, (float_4)max_val);
	}
	
	i_output_time += decimation_factor;
	min_val = DEFAULT_LARGE_NUMBER;
	max_val = -min_val;
      }

      // update counters
      //
      i_cur_time++;
    }
  }

  // close the file
  //
  fclose(fp);

// exit gracefully
//
return(ISIP_NO_ERROR);
}

