// file: /u1/picone/jeida/plot_signal/plot_signal.cc
//
// this is a simple program that prints signal values to stdout
// using a standard interpolation algorithm
//

// system include files
//
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// isip include files
//
#include <math/integral/integral.h>
#include <math/integral/integral_constants.h>

// local include files
//
//#include "plot_signal.h"

// main program
//
main (int argc, char** argv) {

  // parse the command line
  //
  int_4 npix_x = atoi(argv[1]);
  int_4 npix_y = atoi(argv[2]);

  char_1 file_in[999];
  strcpy((char*)file_in, argv[3]);

  char_1 file_out[999];
  strcpy((char*)file_out, argv[4]);

  // variables for command line arguments
  //
  float_4 sample_freq = 16000;
  float_4 start_time = 0.0;
  float_4 end_time;
  int_4 resolution = npix_x / 2;
  int_4 num_bytes = 2;
  int_4 num_chan = 1;
  int_4 chan = 0;
  int_4 buf_size = 64000;

  // create an output buffer
  //
  float_4 out_pts_x[3*resolution];
  float_4 out_pts_y[3*resolution];
  int_4 npts = (int_4)0;

  // open the file to be plotted
  //
  FILE *fp_in = fopen((char*)file_in, "r");

  if (fp_in == NULL) {
    fprintf(stdout, "%s: error opening file [%s]\n", argv[0], file_in);
    return(ISIP_FATAL_ERROR);
  }

  // open the output file
  //
  FILE *fp_out = fopen((char*)file_out, "w");

  if (fp_out == NULL) {
    fprintf(stdout, "%s: error opening file [%s]\n", argv[0], file_out);
    return(ISIP_FATAL_ERROR);
  }

  // compute the end of file
  //
  fseek(fp_in, 0, SEEK_END);
  int_4 file_size = ftell(fp_in);
  end_time = (float_4)file_size / (sample_freq *
				   (float_4)num_bytes * (float_4)num_chan);
  rewind(fp_in);

  // compute an i/o buffer size
  //
  int_4 num_samples = (int_4)((end_time - start_time) * sample_freq);

  if (num_samples*num_chan < buf_size)
    {buf_size = num_samples;}

  // compute the decimation factor and adjust the buf_size
  //
  int_4 decimation_factor = num_samples / resolution;
  if (decimation_factor < (int_4)1)
    {decimation_factor = (int_4)1;}
  if (buf_size % decimation_factor != 0) {
    buf_size += decimation_factor;
    buf_size /= decimation_factor;
    buf_size *= decimation_factor;
  }

  // allocate an i/o buffer
  //
  char_1* buffer = (char_1*)malloc(buf_size*num_chan*num_bytes);

  // initialize counters
  //
  int_4 i_cur_time = (int_4)(start_time * sample_freq);
  int_4 i_end_time = (int_4)(end_time * sample_freq);
  int_4 i_output_time = i_cur_time + decimation_factor - (int_4)1;

  // seek to the starting point in the file
  //
  fseek(fp_in, i_cur_time*num_bytes*num_chan, SEEK_SET);

  // initialize parameters to find min/max
  //
  int_4 sample_val;
  int_4 min_val = 32767;
  int_4 min_loc = (int_4)0;
  int_4 max_val = -min_val;
  int_4 max_loc = (int_4)0;
  float_4 time_scale = 1.0/sample_freq;

  // main loop
  //
  while (i_cur_time < i_end_time) {

    // read data
    //
    fread(buffer, num_chan*num_bytes, buf_size, fp_in);

    // decimate the signal
    //
    for (int_4 i=0; i<buf_size; i++) {

      // compute the sample value
      //
      if (num_bytes == (int_4)2) {
	sample_val = *((int_2*)(buffer + i*(num_chan*num_bytes) +
				chan*num_bytes));
      }
      else if (num_bytes == (int_4)4) {
	sample_val = *((int_4*)(buffer + i*(num_chan*num_bytes) +
				chan*num_bytes));
      }
      else {
	fprintf(stdout,
		"%s: error in sample format [%ld]\n", argv[0], num_bytes);
	return(ISIP_FATAL_ERROR);
      }

      // check for min/max
      //
      if (sample_val < min_val) {
	min_val = sample_val;
	min_loc = i_cur_time;
      }

      if (sample_val > max_val) {
	max_val = sample_val;
	max_loc = i_cur_time;
      }
//      printf("%d %d (%d): %d %d %d %d\n",i_cur_time, i_output_time,
//	     sample_val, min_val, min_loc, max_val, max_loc);

      // output the critical points if necessary
      //
      if ((i_cur_time == i_output_time) && (i_cur_time < i_end_time)) {

	if (min_loc < i_end_time) {
	  out_pts_x[npts] = (min_loc) * time_scale;
	  out_pts_y[npts] = (float_4)min_val;
	  npts++;
	}

	if (max_loc < i_end_time) {
	  out_pts_x[npts] = (max_loc) * time_scale;
	  out_pts_y[npts] = (float_4)max_val;
	  npts++;
	}
	
	i_output_time += decimation_factor;
	min_val = 32767;
	max_val = -min_val;
      }

      // update counters
      //
      i_cur_time++;
    }
  }

  // close the file
  //
  fclose(fp_in);

  // find the maximum magnitude
  //
  float_4 val = 0, max_ampl = 0;

  for (int_4 i=0; i<npts; i++) {
    val = out_pts_y[i];
    if (out_pts_y[i] < 0)
      {val = -out_pts_y[i];}
    if (val > max_ampl)
      {max_ampl = val;}
  }

  fprintf(fp_out, "%f\n", max_ampl);
  fprintf(fp_out, "%f\n", end_time);

  // normalize the data and output
  //
  float_4 scale_x = 1.0 / (end_time - start_time) * (float_4)(npix_x-1);
  float_4 scale_y_1 = -1.0 / max_ampl * (float_4)(npix_y-1) / 2.0;
  float_4 scale_y_2 = npix_y / 2;
  int_4 i_x, i_y;

  fprintf(fp_out, "%ld\n", npts);

  for (i=0; i<npts; i++) {

    i_x = (int_4)rint((out_pts_x[i] - start_time) * scale_x);
    i_y = (int_4)rint((out_pts_y[i] * scale_y_1) + scale_y_2);

    fprintf(fp_out,"%ld %ld\n", i_x,i_y);
  }

  // close files
  fclose(fp_in);
  fclose(fp_out);

  // exit gracefully
  //
  return(ISIP_NO_ERROR);
}

