// file: $JEIDA/src/signal_detector/v2.1/src/sigd_main_2.cc
//

// isip include files
//
#include "signal_detector.h"
#include "signal_detector_constants.h"

// function: adjust_endpoints_cc
//
// arguments:
//  float_4** endpts_p: (input/output) arrays to be adjusted
//  int_4 size_p: (input) number of arrays (typically 2)
//  int_4 num_endpts_p: (input) number of endpoints
//  float_4 gap_time_p: (input) maximum time between two utterances
//				before we become concerned about the endpoints
//  float_4 init_pad_time_p: (input) pad time to be subtracted from the
//				     start time
//  float_4 final_pad_time_p: (input) pad time to be added to the end time
//
// return:
//  an logical_1 value indicating the status
//
// this method adjusts the endpoints to be halfway between two utterances.
// it is useful when utterances come at regular intervals and you know
// approximately where the utterances should be.
//
logical_1 adjust_endpoints_cc(float_4** endpts_p, int_4 size_p,
			      int_4 num_endpts_p, float_4 gap_time_p,
			      float_4 init_pad_time_p,
			      float_4 final_pad_time_p) {

  // declare local variables
  //
  float etime, stime, delta, ntime;

  // loop over all endpts except the first (this code looks backward)
  //
  for (int_4 utt_num=1; utt_num<num_endpts_p; utt_num++) {

    // define an interval between utterances
    //
    stime = (endpts_p[(int_4)0])[utt_num];
    etime = (endpts_p[(int_4)1])[utt_num - (int_4)1];
    delta = stime - etime;
    ntime = etime + (stime - etime) * 0.5;
    
    // if the midpoint between the last utterance and the current
    // utterance is less than 1/2 the gap time, use it
    //
    if ((stime - ntime) < (0.5*gap_time_p)) {
      (endpts_p[(int_4)0])[utt_num] = ntime;
      (endpts_p[(int_4)1])[utt_num - (int_4)1] = ntime;
    }
    // if the gap is significantly larger than the pad time,
    // reduce it to the pad time
    //
    else {
      (endpts_p[(int_4)0])[utt_num] -= init_pad_time_p;
      (endpts_p[(int_4)1])[utt_num - (int_4)1] += final_pad_time_p;
    }
  }      

  // handle the first and last utterances manually
  //
  (endpts_p[(int_4)0])[(int_4)0] -= init_pad_time_p;
  (endpts_p[(int_4)1])[num_endpts_p-(int_4)1] += final_pad_time_p;

  if ((endpts_p[(int_4)0])[(int_4)0] < (float_4)0.0)
    {(endpts_p[(int_4)0])[(int_4)0] = (float_4)0.0;}

  // exit gracefully
  //
  return ISIP_TRUE;
}
