# file: $JEIDA/src/validate/v1.0/src/read_wordlist.tcl
#
# This procedure reads in a new wordlist into the wordlist listbox
# widgets from the $wordlist_filename parameter
#

proc readlist {wordlist_filename} {
    global env
    global maxrownum
    global maxcolnum
    global column_separator
    global blank_separator
    global p

    # open the file
    #
    set the_file [open "$env(JEIDA)/[string trim $wordlist_filename \"]"]
    
    # clear out the listboxes
    #
    for {set colnum 0} {$colnum < $maxcolnum} {incr colnum} {
	$p.word$colnum delete 0 end
    }

    # initialize the rownum and colnum counter variables
    #
    set colnum 0
    set rownum 0
    set need_new_column 0
    while {[gets $the_file line] >= 0 } {
	if { ( [string index $line 0] != "#" ) && ([string index $line 0] != " ") && ([string index $line 0] != "")} {
	    
	    # if the string is readable and not a comment
	    #
	    if { [string first $column_separator $line] >= 0} {
		
		# if the string is the column_separator marker,
		# increment into the new column
		#
		incr colnum +1
		set rownum 0
		set need_new_column 0
		
		# test if maxcolnum is exceeded, exit on error
		#
		if { $colnum == $maxcolnum } {
		    puts "ERROR"
		    puts "Too many columns for display"
		    puts "Change Maxcolnum, or decrease size of wordlist"
		    exit
		}
	    } else {
		if { $need_new_column == 1 } {
		    
		    # test if a new column is needed for row length
		    #
		    incr colnum +1
		    set rownum 0
		    set need_new_column 0
		}
		if { [string first $blank_separator $line] >= 0} {
		    
		    # test if the line is the blank separator
		    #
		    incr rownum +1
		    if {$rownum == $maxrownum} {
			
			# if maxrownum is exceeded, request a new column
			#
			set need_new_column 1
		    }
		    $p.word$colnum insert end " "
		} else {
		    
		    # the entry must be a word for the list, so insert it
		    #
		    $p.word$colnum insert end "$line "
		    incr rownum +1
		    if {$rownum == $maxrownum} {
			
			# if maxrownum is exceeded, request a new column
			#
			set need_new_column 1
		    }
		}
	    }
	}
    }   
    close $the_file
}




